<?php

/***************************************************************
* 
* Mehrsprachigkeit für moziloCMS 
* 
***************************************************************/
class i18n extends Plugin {

    // Übersetzungen für Kategorie- und Inhaltsseiten-Namen
    var $translations_as_properties = null;
    
    // Verfügbare Sprachen
    var $available_languages = null;
    
    /***************************************************************
    * 
    * Gibt den HTML-Code zurück, mit dem die Plugin-Variable ersetzt 
    * wird. Der String-Parameter $value ist Pflicht, kann aber leer 
    * sein.
    * 
    ***************************************************************/
    function getContent($value) {
        global $CAT_REQUEST;
    
        // Verfügbare Übersetzungen initialisieren
        $this->initializeTranslations();

        // Aktuell vorhandene Inhaltsseiten einlesen
        if ($this->settings->get("initialize") == "true") {
            $this->initializeContent();
        }
        
        // Kein Parameter übergeben?
        if ($value == "") {
            return $this->buildErrorMsg("Kein Parameter übergeben. / No parameter given.");
        }

        // Übersetzungen der Kategorie- und Inhaltsseiten-Namen für die gewählte Sprache einlesen (natürlich nur, wenn schon eine gesetzt ist!)
        if (isset($_SESSION["i18n_lang"])) {
            $this->translations_as_properties = $this->getTranslationsAsProperties($_SESSION["i18n_lang"]);
        }

        // Plugin-eigene Platzhalter ersetzen
        if ($value == "mainmenu") {
            return $this->getI18nMainMenu();
        }
        else if ($value == "detailmenu") {
            global $CMS_CONF;
            // Detailmenue (nicht zeigen, wenn Submenues aktiviert sind)
            if ($CMS_CONF->get("usesubmenu") == 0) {
                return $this->getI18nDetailMenu($CAT_REQUEST);
            }
        }
        else if ($value == "sitemap"){
            return $this->getI18nSiteMap(); // danke mdj :)
        }
        else if ($value == "title") {
            return $this->getI18nWebsiteTitle();
        }
        else if ($value == "currentlang") {
            return $_SESSION["i18n_lang"];
        }

        
        // Parameter vom Content trennen
        $content_lang = substr($value, 0, strpos($value, "|"));
        $content = substr($value, strpos($value, "|")+1, strlen($value)-strpos($value, "|"));
        
        // Sprachwechsel-Links (beginnen mit "switch_")
        if (strpos($content_lang, "switch_") === 0) {
            $switchToLang = substr($content_lang, 7);
            $url = $_SERVER['REQUEST_URI'];
            if (count($_GET) < 1) {
                return '<a href="'.$url.'?i18n='.$switchToLang.'" class="i18n i18n_switch_'.$switchToLang.'">'.$content.'</a>';
            }
            else {
                return '<a href="'.$url.'&amp;i18n='.$switchToLang.'" class="i18n i18n_switch_'.$switchToLang.'">'.$content.'</a>';
            }
        }


        // Sprache in der Session schon gesetzt?
        if (isset($_SESSION["i18n_lang"])) {
            // Sprache des aktuellen Inhalts in der Session gesetzt? Dann anzeigen!
            if ($content_lang == $_SESSION["i18n_lang"]) {
                return $content;
            }
        }
        // sonst gegen die Default-Sprache prüfen!
        else {
            // Sprache des aktuellen Inhalts ist die Default-Sprache? Dann anzeigen!
            if ($content_lang == $this->available_languages[0]) {
                return $content;
            }
        }
    } // function getContent
    

    /***************************************************************
    * 
    * Schreibt die aktuelle angeforderte Sprache in die Session.
    * 
    ***************************************************************/
    function initializeTranslations() {
        $this->debug("initializeTranslations");
        if (!isset($this->available_languages)){
            $this->debug("!isset(\$this->available_languages)");
            $this->available_languages = explode(",", $this->settings->get("languages"));
            // Leerzeichen trimmen
            for ($i = 0; $i < count($this->available_languages); $i++) {
                $this->available_languages[$i] = trim($this->available_languages[$i]);
            }
            // angeforderte Sprache vorhanden? Dann setzen
            if (isset($_REQUEST['i18n']) && ($_REQUEST['i18n'] != "") && in_array($_REQUEST['i18n'], $this->available_languages)) {
                $_SESSION["i18n_lang"] = $_REQUEST['i18n'];
            }
            // sonst Standardsprache setzen, wenn Session noch nicht gesetzt
            elseif (!isset($_SESSION["i18n_lang"])) {
                $_SESSION["i18n_lang"] = $this->available_languages[0];
            }
        }
    }
    
    
    /***************************************************************
    * 
    * Gibt die Konfigurationsoptionen als Array zurück.
    * Ist keine Konfiguration nötig, ist dieses Array leer.
    * 
    ***************************************************************/
    function getConfig() {
        $this->debug("getConfig");
        global $ADMIN_CONF;
        $adminlanguage = $ADMIN_CONF->get("language");

        // Verfügbare Übersetzungen initialisieren
        $this->initializeTranslations();

// DEUTSCH
        $config['deDE'] = array();
        $config['deDE']['languages']  = array(
            "type" => "text",
            "description" => "Verfügbare Sprachen (kommasepariert; erste ist der Standard)",
            "regex" => "/^.+$/", // wenigstens ein Zeichen!
            "regex_error" => "Keine Sprachen angegeben."
        );
        // Texteingabefeld für jede konfigurierte Sprache
        foreach ($this->available_languages as $current_lang) {
            $config['deDE']['lang_'.$current_lang] = array(
                "type" => "textarea",
                "cols" => "100",
                "rows" => "10",
                "description" => "Übersetzte Kategorie- und Seitennamen (".$current_lang.")"
            );
        }
        $config['deDE']['initialize'] = array(
            "type" => "checkbox",
            "description" => "Kategorien und Inhaltsseiten neu einlesen <br />(vorhandene&nbsp;Werte&nbsp;bleiben&nbsp;erhalten)"
        );

// ENGLISCH
        $config['enEN'] = array();
        $config['enEN']['languages']  = array(
            "type" => "text",
            "description" => "Available languages (comma-separated; first one is the default)",
            "regex" => "/^.+$/", // wenigstens ein Zeichen!
            "regex_error" => "No langugages specified."
        );
        // Texteingabefeld für jede konfigurierte Sprache
        foreach ($this->available_languages as $current_lang) {
            $config['enEN']['lang_'.$current_lang] = array(
                "type" => "textarea",
                "cols" => "100",
                "rows" => "10",
                "description" => "Translated names of categories and content pages (".$current_lang.")"
            );
        }
        $config['enEN']['initialize'] = array(
            "type" => "checkbox",
            "description" => "Initialize categories and content pages<br />(already&nbsp;set&nbsp;values&nbsp;will&nbsp;not&nbsp;be&nbsp;changed)"
        );
        
        //  Optionen-Array der aktuell im Admin verwendeten Sprache zurückgeben
        if(isset($config[$adminlanguage])) {
            return $config[$adminlanguage];
        }
        // Stellt das Plugin diese Sprache nicht zur Verfügung, werden die englischen Optionen zurückgegeben.
        else {
            return $config['enEN'];
        }
    } // function getConfig    
    
    
    /***************************************************************
    * 
    * Gibt die Plugin-Infos als Array zurück - in dieser 
    * Reihenfolge:
    *   - Name und Version des Plugins
    *   - für moziloCMS-Version
    *   - Kurzbeschreibung
    *   - Name des Autors
    *   - Download-URL
    *   - Platzhalter für die Selectbox
    * 
    ***************************************************************/
    function getInfo() {
        $this->debug("getInfo");
        global $ADMIN_CONF;
        
        $language = $ADMIN_CONF->get("language");
        $pluginname = "<b>i18n</b> (\"Internationalization\") 1.1 (Revision: 160)";
        $cmsversion = "1.12";
        $author = "Arvid Zimmermann";
        $website = "http://software.azett.com";
        
        // Verfügbare Übersetzungen initialisieren
        $this->initializeTranslations();
       
// DEUTSCH
        $pluginvariables_de = array();
        foreach ($this->available_languages as $current_lang) {
            $pluginvariables_de['{i18n|'.$current_lang.'|}'] = 'Mehrsprachiger Inhalt: '.$current_lang;
        }
        $info['deDE'] = array(
            // Plugin-Name + Version
            $pluginname,
            // moziloCMS-Version
            $cmsversion,
            // Kurzbeschreibung nur <span> und <br /> sind erlaubt
            "Mehrsprachigkeit für moziloCMS-Inhalte<br />"
            ."<br />"
            .$this->boldspan("Nutzung")."<br />"
            .$this->codespan("{i18n|de|Text}")." - Text, der nur angezeigt wird, wenn \"de\" die gewählte Sprache ist.<br />"
            ."<br />"
            .$this->codespan("{i18n|switch_de|Linktext}")." - Link (mit beliebigem Linktext) zum Wechseln der Sprache. Hier: Wechsel zu \"de\".<br />"
            .$this->codespan("{i18n|switch_en|Linktext}")." - Anderes Beispiel: Wechsel zu \"en\".<br />"
            ."<br />"
            .$this->codespan("{i18n|mainmenu}")." - Hauptmenü mit den übersetzten Namen von Kategorien (und, bei aktiviertem Submenü, auch Inhaltsseiten).<br />"
            .$this->codespan("{i18n|detailmenu}")." - Detailmenü mit den übersetzten Namen der Inhaltsseiten.<br />"
            .$this->codespan("{i18n|title}")." - Website-Titel mit den übersetzten Namen von Kategorien und Inhaltsseiten.<br />"
            .$this->codespan("{i18n|sitemap}")." - Sitemap mit den übersetzten Namen von Kategorien und Inhaltsseiten.<br />"
            .$this->codespan("{i18n|currentlang}")." - Aktuell gesetzte Sprache, z.B.\"de\".",
            // Name des Autors
            $author,
            // Download-URL
            $website,
            // Platzhalter für die Selectbox in der Editieransicht 
            // - ist das Array leer, erscheint das Plugin nicht in der Selectbox
            $pluginvariables_de
        );

// ENGLISCH
        $pluginvariables_en = array();
        foreach ($this->available_languages as $current_lang) {
            $pluginvariables_en['{i18n|'.$current_lang.'|}'] = 'Multi-language content: '.$current_lang;
        }
        $info['enEN'] = array(
            // Plugin-Name + Version
            $pluginname,
            // moziloCMS-Version
            $cmsversion,
            // Kurzbeschreibung nur <span> und <br /> sind erlaubt
            "Multi-language content for moziloCMS<br />"
            ."<br />"
            .$this->boldspan("Usage")."<br />"
            .$this->codespan("{i18n|en|Text}")." - Text which is only shown if \"en\" is selected language.<br />"
            ."<br />"
            .$this->codespan("{i18n|switch_en|link text}")." - Link (with any link text) to switch the language. Here: Switch to \"en\".<br />"
            .$this->codespan("{i18n|switch_de|link text}")." - Another example: Switch to \"de\".<br />"
            ."<br />"
            .$this->codespan("{i18n|mainmenu}")." - Main menu with translated names of categories (and, with activated submenu, content pages).<br />"
            .$this->codespan("{i18n|detailmenu}")." - Detail menu with translated names of the content pages.<br />"
            .$this->codespan("{i18n|title}")." - Website title with translated names of categories and content pages."
            .$this->codespan("{i18n|sitemap}")." - Sitemap with translated names of categories and content pages."
            .$this->codespan("{i18n|currentlang}")." - Currently set language, i.e. \"en\".",
            // Name des Autors
            $author,
            // Download-URL
            $website,
            // Platzhalter für die Selectbox in der Editieransicht 
            // - ist das Array leer, erscheint das Plugin nicht in der Selectbox
            $pluginvariables_en
        );
        
        // Info-Array der aktuell im Admin verwendeten Sprache zurückgeben
        if(isset($info[$language])) {
            return $info[$language];
        }
        // Stellt das Plugin diese Sprache nicht zur Verfügung, werden die englischen Optionen zurückgegeben.
        else {
            return $info['enEN'];
        }
    } // function getInfo
    

    /***************************************************************
    * 
    * Aufbau des Hauptmenues, Rueckgabe als String
    * 
    * Diese Funktion wurde aus dem CMS-Standard 1.12.beta3 kopiert 
    * und angepaßt.
    * 
    ***************************************************************/
    function getI18nMainMenu() {
        $this->debug("getI18nMainMenu");
        global $CONTENT_DIR_REL;
        global $CAT_REQUEST;
        global $PAGE_REQUEST;
        global $specialchars;
        global $CMS_CONF;
        global $language;
        global $syntax;
        global $URL_BASE;
        global $EXT_LINK;

        $mainmenu = "<ul class=\"mainmenu\">";
        // Kategorien-Verzeichnis einlesen
        $categoriesarray = getDirContentAsArray($CONTENT_DIR_REL, false, false);
        // Jedes Element des Arrays ans Menue anhaengen
        foreach ($categoriesarray as $currentcategory) {

            // i18n: Standard-Namen durch Übersetzung ersetzen, wenn vorhanden
            $i18n_linktext = $this->getTranslationForCat($currentcategory);
            // /i18n
            
            # Mod Rewrite
            $url = $URL_BASE."index.php?cat=".substr($currentcategory,3);
            if($CMS_CONF->get("modrewrite") == "true") {
                $url = $URL_BASE.substr($currentcategory,3).".html";
            }
            if(substr($currentcategory,-(strlen($EXT_LINK))) == $EXT_LINK) {
               $mainmenu .= '<li class="mainmenu">'.menuLink($currentcategory,"menu")."</li>";
            }
            // Wenn die Kategorie keine Contentseiten hat, zeige sie nicht an
            $catcontent = getDirContentAsArray($CONTENT_DIR_REL.$currentcategory, true, false);
            if (($catcontent == "") || count($catcontent) == 0) {
                $mainmenu .= "";
            }
            // Aktuelle Kategorie als aktiven Menuepunkt anzeigen...
            elseif ($currentcategory == $CAT_REQUEST) {
                $mainmenu .= "<li class=\"mainmenu\">".
                    "<a href=\"".$url."\" class=\"menuactive\"".
                    $syntax->getTitleAttribute($language->getLanguageValue1("tooltip_link_category_1", $i18n_linktext)).
                    ">".$i18n_linktext."</a>";
                if ($CMS_CONF->get("usesubmenu") > 0) {
                    $mainmenu .= $this->getI18nDetailMenu($currentcategory);
                }
                $mainmenu .= "</li>";
            }
            // ...alle anderen als normalen Menuepunkt.
            else {
                $mainmenu .= "<li class=\"mainmenu\">".
                    "<a href=\"".$url."\" class=\"menu\"".
                     $syntax->getTitleAttribute($language->getLanguageValue1("tooltip_link_category_1", $i18n_linktext)).
                     ">".$i18n_linktext."</a>";
                if ($CMS_CONF->get("usesubmenu") == 2) {
                    $mainmenu .= $this->getI18nDetailMenu($currentcategory);
                }
                $mainmenu .= "</li>";
            }
        }
        // Rueckgabe des Menues
        return $mainmenu . "</ul>";
    }

    /***************************************************************
    * 
    * Aufbau des Detailmenues, Rueckgabe als String
    * 
    * Diese Funktion wurde aus dem CMS-Standard 1.12.beta3 kopiert 
    * und angepaßt.
    * 
    ***************************************************************/
    function getI18nDetailMenu($cat){
        $this->debug("getI18nDetailMenu");
        global $ACTION_REQUEST;
        global $QUERY_REQUEST;
        global $CONTENT_DIR_REL;
        global $CAT_REQUEST;
        global $PAGE_REQUEST;
        global $EXT_DRAFT;
        global $language;
        global $specialchars;
        global $CMS_CONF;
        global $syntax;
        global $URL_BASE;
        global $EXT_LINK;
        global $CHARSET;

        if ($CMS_CONF->get("usesubmenu") > 0)
            $cssprefix = "submenu";
        else
            $cssprefix = "detailmenu";

        # Mod Rewrite
        $url_draft = $URL_BASE."index.php?cat=".substr($cat,3)."&amp;page=".substr($PAGE_REQUEST, 3)."&amp;";
        $modrewrite_dumy = NULL;
        if($CMS_CONF->get("modrewrite") == "true") {
            $url_draft = $URL_BASE.substr($cat,3)."/".substr($PAGE_REQUEST, 3).".html?";
            $modrewrite_dumy = ".html";
        }
        $detailmenu = "<ul class=\"detailmenu\">";
        // Sitemap
        if (($ACTION_REQUEST == "sitemap") && ($CMS_CONF->get("usesubmenu") == 0))
            $detailmenu .= "<li class=\"detailmenu\"><a href=\"".$URL_BASE."index.php".$modrewrite_dumy."?action=sitemap\" class=\"".$cssprefix."active\">".$language->getLanguageValue0("message_sitemap_0")."</a></li>";
        // Suchergebnis
        elseif (($ACTION_REQUEST == "search") && ($CMS_CONF->get("usesubmenu") == 0))
            $detailmenu .= "<li class=\"detailmenu\"><a href=\"".$URL_BASE."index.php".$modrewrite_dumy."?action=search&amp;query=".$specialchars->replaceSpecialChars($QUERY_REQUEST, false)."\" class=\"".$cssprefix."active\">".$language->getLanguageValue1("message_searchresult_1", $specialchars->getHtmlEntityDecode($QUERY_REQUEST))."</a></li>";
        // Entwurfsansicht
        elseif (($ACTION_REQUEST == "draft") && ($CMS_CONF->get("usesubmenu") == 0))
            $detailmenu .= "<li class=\"detailmenu\"><a href=\"".$url_draft."action=draft\" class=\"".$cssprefix."active\">".pageToName($PAGE_REQUEST.$EXT_DRAFT, false)." (".$language->getLanguageValue0("message_draft_0").")</a></li>";
        // "ganz normales" Detailmenue einer Kategorie
        else {
            // Content-Verzeichnis der aktuellen Kategorie einlesen
            $contentarray = getDirContentAsArray($CONTENT_DIR_REL.$cat, true, false);

            // Kategorie, die nur versteckte Seiten enthaelt: kein Detailmenue zeigen
            if ($contentarray == "") {
                return "";
            }

            // Jedes Element des Arrays ans Menue anhaengen
            foreach ($contentarray as $currentcontent) {
                // Inhaltsseite nicht anzeigen, wenn sie genauso heißt wie die Kategorie
                if ($CMS_CONF->get("hidecatnamedpages") == "true") {
                    if(substr($currentcontent, 3, strlen($currentcontent) - 7) == substr($cat, 3) and substr($currentcontent,-(strlen($EXT_LINK))) != $EXT_LINK) {
                        // Wenn es in der Kategorie nur diese eine (dank hidecatnamedpages eh nicht angezeigte) Seite gibt,
                        // dann gib als Detailmenue gleich einen Leerstring zurueck
                        if (count($contentarray) == 1) {
                            return "";
                        } 
                        // ...ansonsten auf zur naechsten Inhaltsseite!
                        else {
                            continue;
                        }
                    }
                }

                // i18n: Standard-Namen durch Übersetzung ersetzen, wenn vorhanden
                $i18n_pagetext = $this->getTranslationForPage($cat, $currentcontent, true);
                $i18n_cattext = $this->getTranslationForCat($cat);
                // /i18n

                # Mod Rewrite
                $url = $URL_BASE."index.php?cat=".substr($cat,3)."&amp;page=".substr($currentcontent, 3, strlen($currentcontent) - 7);
                if($CMS_CONF->get("modrewrite") == "true") {
                    $url = $URL_BASE.substr($cat,3)."/".substr($currentcontent, 3, strlen($currentcontent) - 7).".html";
                }
                // Aktuelle Inhaltsseite als aktiven Menuepunkt anzeigen...
                if (
                    ($CAT_REQUEST == $cat) // aktive Kategorie
                    && (substr($currentcontent, 0, strlen($currentcontent) - 4) == $PAGE_REQUEST) // aktive Seite
                    && (substr($currentcontent, -(strlen($EXT_LINK))) != $EXT_LINK) // aktive Seite
                ) {
                    $detailmenu .= "<li class=\"detailmenu\"><a href=\"".$url.
                                                    "\" class=\"".$cssprefix."active\"".
                                                    $syntax->getTitleAttribute($language->getLanguageValue2("tooltip_link_page_2", $i18n_pagetext, $i18n_cattext)).
                                                    ">".
                                                    $i18n_pagetext.
                                                    "</a></li>";
                }
                // ...alle anderen als normalen Menuepunkt.
                else {
                    if(substr($currentcontent,-(strlen($EXT_LINK))) == $EXT_LINK) {
                        $detailmenu .= '<li class="detailmenu">'.menuLink($currentcontent,$cssprefix)."</li>";
                    } else {
                        $detailmenu .= "<li class=\"detailmenu\"><a href=\"".$url.
                                                    "\" class=\"".$cssprefix."\"".
                                                    $syntax->getTitleAttribute($language->getLanguageValue2("tooltip_link_page_2", $i18n_pagetext, $i18n_cattext)).
                                                    ">".
                                                    $i18n_pagetext.
                                                    "</a></li>";
                    }
                }
            }
        }
        // Rueckgabe des Menues
        return $detailmenu . "</ul>";
    }
    
    /***************************************************************
    * 
    * Aufbau der Sitemap, Rueckgabe als String
    * 
    * Diese Funktion wurde aus dem CMS-Standard 1.12.beta4 kopiert 
    * und angepaßt.
    * 
    ***************************************************************/
    function getI18nSiteMap() {
        global $language;
        global $CMS_CONF;
        global $CatPage;
        global $EXT_PAGE;
        global $EXT_HIDDEN;

        $include_pages = array($EXT_PAGE);
        if($CMS_CONF->get("showhiddenpagesinsitemap") == "true") {
            $include_pages = array($EXT_PAGE,$EXT_HIDDEN);
        }

        //$sitemap = "<h1>".$language->getLanguageValue0("message_sitemap_0")."</h1>"
        $sitemap = "<div class=\"sitemap\">";
        // Kategorien-Verzeichnis einlesen
        $categoriesarray = $CatPage->get_CatArray(false, false, $include_pages);
        // Jedes Element des Arrays an die Sitemap anhaengen
        foreach ($categoriesarray as $currentcategory) {
            $i18n_cat = $this->getTranslationForCat("___".urldecode($currentcategory));
            //$sitemap .= "<h2>".$CatPage->get_HrefText($currentcategory,false)."</h2><ul>";
            $sitemap .= "<h2>".$i18n_cat."</h2><ul>";
            // Inhaltsseiten-Verzeichnis einlesen
            $contentarray = $CatPage->get_PageArray($currentcategory,$include_pages,true);
            // Alle Inhaltsseiten der aktuellen Kategorie auflisten...
            // Jedes Element des Arrays an die Sitemap anhaengen
            foreach ($contentarray as $currentcontent) {
                $url = $CatPage->get_Href($currentcategory,$currentcontent);
                $urltext = $CatPage->get_HrefText($currentcategory,$currentcontent);
                $i18n_page = html_entity_decode($this->getTranslationForPage("___".urldecode($currentcategory), urldecode("___".$urltext.".___"), false));
                $titel = $language->getLanguageValue2("tooltip_link_page_2", $CatPage->get_HrefText($currentcategory,$currentcontent), $CatPage->get_HrefText($currentcategory,false));
                $sitemap .= "<li>".$CatPage->create_LinkTag($url,$i18n_page,false,$titel)."</li>";
            }
            $sitemap .= "</ul>";
        }
        $sitemap .= "</div>";
        // Rueckgabe der Sitemap
        return $sitemap;
    }

    
    /***************************************************************
    * 
    * Hilfsfunktion: Textarea-Inhalt als Properties-Objekt zurückgeben
    * 
    ***************************************************************/
    function getTranslationsAsProperties($lang) {
        $this->debug("getTranslationsAsProperties");
        $translations_as_array = explode("<br />", $this->settings->get("lang_".$lang));
        // Übersetzungen einlesen
        $translations_as_properties = new SimpleProperties();
        foreach ($translations_as_array as $currentline) {
            // Zeile am ersten "=" aufsplitten
            $equalpos = strpos($currentline, "=");
            if ($equalpos > 0) {
                $currentvalue = trim(substr($currentline, $equalpos+1, strlen($currentline)-$equalpos));
                $currentkey = trim(substr($currentline, 0, $equalpos));
                $translations_as_properties->set($currentkey, $currentvalue);
            }
        }
        return $translations_as_properties;
    }

    
    /***************************************************************
    * 
    * Hilfsfunktion: Properties-Objekt in die Plugin-Settings speichern
    * 
    ***************************************************************/
    function setTranslationsFromProperties($translations, $lang) {
        $this->debug("setTranslationsFromProperties");
        $translations_as_array = $translations->toArray();
        $translations = "";
        foreach ($translations_as_array as $currentkey => $currentvalue) {
            if ($currentkey != "readonly") {
                $translations = $translations . $currentkey . " = " . $currentvalue . "<br />";
            }
        }
        $this->settings->set("lang_".$lang, $translations);
    }

    
    /***************************************************************
    * 
    * Liest Kategorien und Inhaltsseiten in die Plugin-Config ein.
    * Bereits vorhandene Elemente werden dabei nicht geändert.
    * 
    ***************************************************************/
    function initializeContent() {
        $this->debug("initializeContent");
        global $CONTENT_DIR_REL;
        global $specialchars;
        
        foreach ($this->available_languages as $current_lang) {
            $translations_as_properties = $this->getTranslationsAsProperties($current_lang);

            $categoriesarray = getDirContentAsArray($CONTENT_DIR_REL, false, false);
            foreach ($categoriesarray as $currentcategory) {
                // Kategorie
                $keyToCheck = $specialchars->getHtmlEntityDecode(catToName($currentcategory, false));
                if (!$translations_as_properties->keyExists($keyToCheck)) {
                        $translations_as_properties->set($keyToCheck, "");
                }
                // zugehörige Inhaltsseiten
                $contentarray = getDirContentAsArray($CONTENT_DIR_REL.$currentcategory, true, true, true);
                if ($contentarray != 0) {
                    foreach ($contentarray as $currentcontent) {
                        $keyToCheck = $specialchars->getHtmlEntityDecode(catToName($currentcategory, false)) . ":" . $specialchars->getHtmlEntityDecode(pageToName($currentcontent, false));
                        if (!$translations_as_properties->keyExists($keyToCheck)) {
                            $translations_as_properties->set($keyToCheck, "");
                        }
                    }
                }
                $this->setTranslationsFromProperties($translations_as_properties, $current_lang);
            }
        }
        $this->settings->set("initialize", "false");
    }


    /***************************************************************
    * 
    * Rueckgabe des Website-Titels
    * 
    * Diese Funktion wurde aus dem CMS-Standard 1.12.beta3 kopiert 
    * und angepaßt.
    * 
    ***************************************************************/
    function getI18nWebsiteTitle() {
        $this->debug("getI18nWebsiteTitle");
        global $CMS_CONF;
        global $specialchars;

        global $ACTION_REQUEST;
        global $WEBSITE_NAME;
        global $CAT_REQUEST;
        global $PAGE_REQUEST;
        $websitetitle = $WEBSITE_NAME;
        $cattitle = $this->getTranslationForCat($CAT_REQUEST);
        $pagetitle = $this->getTranslationForPage($CAT_REQUEST, $PAGE_REQUEST.".txt");

        // Für Suche und Sitemap: Standard-Titel zurückgeben
        if (($ACTION_REQUEST == "sitemap") || ($ACTION_REQUEST == "search")) {
            // TODO: Hier könnte man auch noch Übersetzungen abfragen!
            return getWebsiteTitle();
        }

        $title = $specialchars->rebuildSpecialChars($CMS_CONF->get("titlebarformat"),true,true);
        $sep = $specialchars->rebuildSpecialChars($CMS_CONF->get("titlebarseparator"),true,true);
        $title = preg_replace('/{WEBSITE}/', $websitetitle, $title);
        if ($cattitle == "") {
            $title = preg_replace('/{CATEGORY}/', "", $title);
        }
        else {
            $title = preg_replace('/{CATEGORY}/', $cattitle, $title);
        }
        $title = preg_replace('/{PAGE}/', $pagetitle, $title);
        $title = preg_replace('/{SEP}/', $sep, $title);
        return $title;
    }

    
    /***************************************************************
    * 
    * Standard-Kategorienamen durch Übersetzung ersetzen, wenn vorhanden
    * 
    ***************************************************************/
    function getTranslationForCat($cat) {
        $this->debug("getTranslationForCat(".$cat.")");
        global $specialchars;
        global $CHARSET;
        $i18n_catToCheck = $specialchars->getHtmlEntityDecode(catToName($cat, false, false));
        if (($this->translations_as_properties != null) && $this->translations_as_properties->keyExists($i18n_catToCheck) && ($this->translations_as_properties->get($i18n_catToCheck) != "")) {
            return htmlentities($this->translations_as_properties->get($i18n_catToCheck),ENT_COMPAT,$CHARSET);
        } else {
            return catToName($cat, false);
        }
    }

    
    /***************************************************************
    * 
    * Standard-Inhaltsseitennamen durch Übersetzung ersetzen, wenn vorhanden
    * 
    ***************************************************************/
    function getTranslationForPage($cat, $page) {
        $this->debug("getTranslationForPage(".$cat.", ".$page.")");
        global $specialchars;
        global $CHARSET;
        $i18n_pageToCheck = $specialchars->getHtmlEntityDecode(catToName($cat, false, false)) . ":" . $specialchars->getHtmlEntityDecode(pageToName($page, false));
        if (($this->translations_as_properties != null) && $this->translations_as_properties->keyExists($i18n_pageToCheck) && ($this->translations_as_properties->get($i18n_pageToCheck) != "")) {
            return htmlentities($this->translations_as_properties->get($i18n_pageToCheck),ENT_COMPAT,$CHARSET);
        } else {
            return pageToName($page, false);
        }
    }
    
    /***************************************************************
    * 
    * Hilfsfunktion: Gibt Fehlermeldung aus
    * 
    ***************************************************************/
    function buildErrorMsg($msg) {
        return '<span class="deadlink">Fehler im Plugin ankerlink: '.$msg.'</span>';
    }
    
    /***************************************************************
    * 
    * Hilfsfunktion: Text in Code-Span packen
    * 
    ***************************************************************/
    function codespan($content) {
        return "<span style=\"font-family:monospace;margin:3px 3px; background-color:#eee;border:1px dotted #aaa;\">".$content."</span>";
    }

    /***************************************************************
    * 
    * Hilfsfunktion: Text in Bold-Span packen
    * 
    ***************************************************************/
    function boldspan($content) {
        return "<span style=\"font-weight:bold;\">".$content."</span>";
    }
    
    /***************************************************************
    * 
    * Hilfsfunktion: Debug-Ausgabe
    * 
    ***************************************************************/
    function debug($msg) {
        if (false) { // Zum Aktivieren aus "false" "true" machen!
            echo $msg."<br />";
        }
    }
    
} // class i18n

/***************************************************************
* 
* Innere Properties-Klasse
* 
***************************************************************/    
class SimpleProperties
{
    private $data = array();

    public function set($name, $value) {
        $this->data[$name] = $value;
    }

    public function get($name) {
        if (array_key_exists($name, $this->data)) {
            return $this->data[$name];
        }
        return null;
    }

    public function __isset($name) {
        return isset($this->data[$name]);
    }

    public function __unset($name) {
        unset($this->data[$name]);
    }

    public function keyExists($name) {
        return array_key_exists($name, $this->data);
    }
    
    public function toArray() {
        return $this->data;
    }
} // class SimpleProperties

?>